@@ -310,7 +310,7 @@ static int process_snapshot(const char *path, const u8 *uuid, u64 ctransid,
 		else
 			ret = PTR_ERR(parent_subvol);
 		uuid_unparse(parent_uuid, uuid_str);
-		error("snapshot: cannot find parent subvolume %s", uuid_str);
+		error("snapshot receive: cannot find parent subvolume %s", uuid_str);
 		goto out;
 	}
 
@@ -1555,7 +1555,8 @@ static int do_receive(struct btrfs_receive *rctx, const char *tomnt,
 			error("failed to chdir to / after chroot: %m");
 			goto out;
 		}
-		fprintf(stderr, "Chroot to %s\n", dest_dir_full_path);
+		if (bconf.verbose > BTRFS_BCONF_QUIET)
+			fprintf(stderr, "Chroot to %s\n", dest_dir_full_path);
 		rctx->root_path = strdup("/");
 		rctx->dest_dir_path = rctx->root_path;
 	} else {
@@ -1782,7 +1783,12 @@ static int cmd_receive(const struct cmd_struct *cmd, int argc, char **argv)
 	tomnt = argv[optind];
 
 	if (fromfile[0]) {
-		receive_fd = open(fromfile, O_RDONLY | O_NOATIME);
+		int flags = O_RDONLY;
+
+		if (!dump)
+			flags |= O_NOATIME;
+
+		receive_fd = open(fromfile, flags);
 		if (receive_fd < 0) {
 			error("cannot open %s: %m", fromfile);
 			goto out;
