@@ -3,7 +3,7 @@
 # Verify that subvolume sync waits until the subvolume is cleaned and does not
 # crash at the end
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
@@ -18,12 +18,12 @@ run_check $SUDO_HELPER chmod a+rw "$TEST_MNT"
 cd "$TEST_MNT"
 
 for i in `seq 5`; do
-	run_check dd if=/dev/zero of=file$i bs=1M count=10
+	run_check dd if=/dev/zero of="file$i" bs=1M count=10
 done
 
 # 128 is minimum
 for sn in `seq 130`;do
-	run_check $SUDO_HELPER "$TOP/btrfs" subvolume snapshot . snap$sn
+	run_check $SUDO_HELPER "$TOP/btrfs" subvolume snapshot . "snap$sn"
 	for i in `seq 10`; do
 		run_check dd if=/dev/zero of="snap$sn/file$i" bs=1M count=1
 	done
